@@ -17,34 +17,35 @@
  */
 package org.apache.hadoop.hdfs.server.namenode;
 
-  import static org.junit.Assert.assertEquals;
-  import static org.junit.Assert.assertTrue;
-  import static org.junit.Assert.fail;
-
-  import org.apache.commons.logging.Log;
-  import org.apache.commons.logging.LogFactory;
-  import org.apache.hadoop.conf.Configuration;
-  import org.apache.hadoop.fs.ContentSummary;
-  import org.apache.hadoop.fs.Path;
-  import org.apache.hadoop.fs.StorageType;
-  import org.apache.hadoop.hdfs.DFSConfigKeys;
-  import org.apache.hadoop.hdfs.DFSTestUtil;
-  import org.apache.hadoop.hdfs.DistributedFileSystem;
-  import org.apache.hadoop.hdfs.MiniDFSCluster;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
+import org.apache.hadoop.conf.Configuration;
+import org.apache.hadoop.fs.ContentSummary;
+import org.apache.hadoop.fs.Path;
+import org.apache.hadoop.fs.SafeModeAction;
+import org.apache.hadoop.fs.StorageType;
+import org.apache.hadoop.hdfs.DFSConfigKeys;
+import org.apache.hadoop.hdfs.DFSTestUtil;
+import org.apache.hadoop.hdfs.DistributedFileSystem;
+import org.apache.hadoop.hdfs.MiniDFSCluster;
 import org.apache.hadoop.hdfs.protocol.DSQuotaExceededException;
 import org.apache.hadoop.hdfs.protocol.HdfsConstants;
 import org.apache.hadoop.hdfs.protocol.QuotaByStorageTypeExceededException;
 import org.apache.hadoop.hdfs.server.namenode.snapshot.SnapshotTestHelper;
-  import org.apache.hadoop.test.GenericTestUtils;
+import org.apache.hadoop.test.GenericTestUtils;
 import org.junit.After;
-  import org.junit.Before;
-  import org.junit.Test;
+import org.junit.Before;
+import org.junit.Test;
+
+import java.io.IOException;
 
-  import java.io.IOException;
 import static org.hamcrest.CoreMatchers.containsString;
 import static org.hamcrest.CoreMatchers.allOf;
 import static org.hamcrest.CoreMatchers.is;
 import static org.junit.Assert.assertThat;
+import static org.junit.Assert.assertEquals;
+import static org.junit.Assert.assertTrue;
+import static org.junit.Assert.fail;
 
 public class TestQuotaByStorageType {
 
@@ -58,7 +59,8 @@ public class TestQuotaByStorageType {
   private DistributedFileSystem dfs;
   private FSNamesystem fsn;
 
-  protected static final Log LOG = LogFactory.getLog(TestQuotaByStorageType.class);
+  protected static final Logger LOG =
+      LoggerFactory.getLogger(TestQuotaByStorageType.class);
 
   @Before
   public void setUp() throws Exception {
@@ -722,9 +724,9 @@ public class TestQuotaByStorageType {
     assertEquals(file1Len, ssdConsumed);
 
     // Restart the namenode with checkpoint to make sure fsImage is correct
-    dfs.setSafeMode(HdfsConstants.SafeModeAction.SAFEMODE_ENTER);
+    dfs.setSafeMode(SafeModeAction.ENTER);
     dfs.saveNamespace();
-    dfs.setSafeMode(HdfsConstants.SafeModeAction.SAFEMODE_LEAVE);
+    dfs.setSafeMode(SafeModeAction.LEAVE);
     cluster.restartNameNode(true);
     refreshClusterState();
 
@@ -944,4 +946,4 @@ public class TestQuotaByStorageType {
         testDirNode.asDirectory().getDirectoryWithQuotaFeature().getQuota()
             .getTypeSpace(StorageType.SSD));
   }
-}
\ No newline at end of file
+}
