@@ -26,16 +26,21 @@ import org.apache.hadoop.yarn.server.resourcemanager.scheduler.capacity.Capacity
 import org.apache.hadoop.yarn.server.resourcemanager.scheduler.fair.FairScheduler;
 import org.apache.hadoop.yarn.server.resourcemanager.scheduler.fair.FairSchedulerConfiguration;
 
+
+
+import org.apache.hadoop.yarn.server.resourcemanager.scheduler.fair
+    .allocationfile.AllocationFileQueue;
+import org.apache.hadoop.yarn.server.resourcemanager.scheduler.fair
+    .allocationfile.AllocationFileWriter;
+import org.junit.After;
 import org.junit.runner.RunWith;
 import org.junit.runners.Parameterized;
 
 import java.io.File;
-import java.io.FileWriter;
 import java.io.IOException;
-import java.io.PrintWriter;
-import java.util.ArrayList;
+import java.util.Arrays;
 import java.util.Collection;
-import java.util.List;
+import java.util.stream.Collectors;
 
 @RunWith(Parameterized.class)
 public abstract class ParameterizedSchedulerTestBase {
@@ -50,11 +55,8 @@ public abstract class ParameterizedSchedulerTestBase {
 
   @Parameterized.Parameters(name = "{0}")
   public static Collection<Object[]> getParameters() {
-    List<Object[]> params = new ArrayList<>(SchedulerType.values().length);
-    for (SchedulerType type : SchedulerType.values()) {
-      params.add(new Object[]{ type });
-    }
-    return params;
+    return Arrays.stream(SchedulerType.values()).map(
+        type -> new Object[]{type}).collect(Collectors.toList());
   }
 
   private SchedulerType schedulerType;
@@ -75,42 +77,47 @@ public abstract class ParameterizedSchedulerTestBase {
 
     schedulerType = type;
     switch (schedulerType) {
-      case FAIR:
-        configureFairScheduler(conf);
-        scheduler = new FairScheduler();
-        conf.set(YarnConfiguration.RM_SCHEDULER,
-            FairScheduler.class.getName());
-        break;
-      case CAPACITY:
-        scheduler = new CapacityScheduler();
-        ((CapacityScheduler)scheduler).setConf(conf);
-        conf.set(YarnConfiguration.RM_SCHEDULER,
-            CapacityScheduler.class.getName());
-        break;
-      default:
-        throw new IllegalArgumentException("Invalid type: " + type);
+    case FAIR:
+      configureFairScheduler(conf);
+      scheduler = new FairScheduler();
+      conf.set(YarnConfiguration.RM_SCHEDULER,
+          FairScheduler.class.getName());
+      break;
+    case CAPACITY:
+      scheduler = new CapacityScheduler();
+      ((CapacityScheduler)scheduler).setConf(conf);
+      conf.set(YarnConfiguration.RM_SCHEDULER,
+          CapacityScheduler.class.getName());
+      break;
+    default:
+      throw new IllegalArgumentException("Invalid type: " + type);
     }
   }
 
-  private void configureFairScheduler(YarnConfiguration conf) throws IOException {
+  protected void configureFairScheduler(YarnConfiguration configuration) {
     // Disable queueMaxAMShare limitation for fair scheduler
-    PrintWriter out = new PrintWriter(new FileWriter(FS_ALLOC_FILE));
-    out.println("<?xml version=\"1.0\"?>");
-    out.println("<allocations>");
-    out.println("<queueMaxAMShareDefault>-1.0</queueMaxAMShareDefault>");
-    out.println("<defaultQueueSchedulingPolicy>fair</defaultQueueSchedulingPolicy>");
-    out.println("<queue name=\"root\">");
-    out.println("  <schedulingPolicy>drf</schedulingPolicy>");
-    out.println("  <weight>1.0</weight>");
-    out.println("  <fairSharePreemptionTimeout>100</fairSharePreemptionTimeout>");
-    out.println("  <minSharePreemptionTimeout>120</minSharePreemptionTimeout>");
-    out.println("  <fairSharePreemptionThreshold>.5</fairSharePreemptionThreshold>");
-    out.println("</queue>");
-    out.println("</allocations>");
-    out.close();
+    AllocationFileWriter.create()
+        .fairDefaultQueueSchedulingPolicy()
+        .disableQueueMaxAMShareDefault()
+        .addQueue(new AllocationFileQueue.Builder("root")
+            .schedulingPolicy("drf")
+            .weight(1.0f)
+            .fairSharePreemptionTimeout(100)
+            .minSharePreemptionTimeout(120)
+            .fairSharePreemptionThreshold(.5)
+            .build())
+        .writeToFile(FS_ALLOC_FILE);
 
-    conf.set(FairSchedulerConfiguration.ALLOCATION_FILE, FS_ALLOC_FILE);
-    conf.setLong(FairSchedulerConfiguration.UPDATE_INTERVAL_MS, 10);
+    configuration.set(FairSchedulerConfiguration.ALLOCATION_FILE,
+        FS_ALLOC_FILE);
+    configuration.setLong(FairSchedulerConfiguration.UPDATE_INTERVAL_MS, 10);
+  }
+
+  @After
+  public void tearDown() {
+    if (schedulerType == SchedulerType.FAIR) {
+      (new File(FS_ALLOC_FILE)).delete();
+    }
   }
 
   public SchedulerType getSchedulerType() {
