@@ -30,6 +30,7 @@ import org.apache.hadoop.yarn.service.api.records.PlacementScope;
 import org.apache.hadoop.yarn.service.api.records.PlacementType;
 import org.apache.hadoop.yarn.service.api.records.Resource;
 import org.apache.hadoop.yarn.service.api.records.Service;
+import org.apache.hadoop.yarn.service.api.records.ServiceState;
 import org.apache.hadoop.yarn.service.exceptions.RestApiErrorMessages;
 import org.junit.Assert;
 import org.junit.BeforeClass;
@@ -44,10 +45,13 @@ import java.util.Collection;
 import java.util.Collections;
 import java.util.List;
 
+import static org.assertj.core.api.Assertions.assertThat;
+import static org.apache.hadoop.test.LambdaTestUtils.intercept;
 import static org.apache.hadoop.yarn.service.conf.RestApiConstants.DEFAULT_UNLIMITED_LIFETIME;
 import static org.apache.hadoop.yarn.service.exceptions.RestApiErrorMessages.*;
 import static org.junit.Assert.assertEquals;
 import static org.junit.Assert.assertNotNull;
+import static org.junit.Assert.fail;
 
 /**
  * Test for ServiceApiUtil helper methods.
@@ -267,7 +271,7 @@ public class TestServiceApiUtil extends ServiceTestUtils {
       Assert.fail(NO_EXCEPTION_PREFIX + e.getMessage());
     }
 
-    assertEquals(app.getLifetime(), DEFAULT_UNLIMITED_LIFETIME);
+    assertThat(app.getLifetime()).isEqualTo(DEFAULT_UNLIMITED_LIFETIME);
   }
 
   private static Resource createValidResource() {
@@ -552,33 +556,11 @@ public class TestServiceApiUtil extends ServiceTestUtils {
     // Set the scope
     pc.setScope(PlacementScope.NODE);
 
-    try {
-      ServiceApiUtil.validateAndResolveService(app, sfs, CONF_DNS_ENABLED);
-      Assert.fail(EXCEPTION_PREFIX + "constraint with no tag(s)");
-    } catch (IllegalArgumentException e) {
-      assertEquals(String.format(
-          RestApiErrorMessages.ERROR_PLACEMENT_POLICY_CONSTRAINT_TAGS_NULL,
-          "CA1 ", "comp-a"), e.getMessage());
-    }
-
-    // Set a target tag - but an invalid one
-    pc.setTargetTags(Collections.singletonList("comp-invalid"));
-
-    try {
-      ServiceApiUtil.validateAndResolveService(app, sfs, CONF_DNS_ENABLED);
-      Assert.fail(EXCEPTION_PREFIX + "constraint with invalid tag name");
-    } catch (IllegalArgumentException e) {
-      assertEquals(
-          String.format(
-              RestApiErrorMessages.ERROR_PLACEMENT_POLICY_TAG_NAME_NOT_SAME,
-              "comp-invalid", "comp-a", "comp-a", "comp-a"),
-          e.getMessage());
-    }
-
-    // Set valid target tags now
+    // Target tag is optional.
     pc.setTargetTags(Collections.singletonList("comp-a"));
 
-    // Finally it should succeed
+    // Validation can succeed for any arbitrary target, only scheduler knows
+    // if the target tag is valid.
     try {
       ServiceApiUtil.validateAndResolveService(app, sfs, CONF_DNS_ENABLED);
     } catch (IllegalArgumentException e) {
@@ -636,6 +618,14 @@ public class TestServiceApiUtil extends ServiceTestUtils {
     } catch (IllegalArgumentException e) {
       Assert.fail(NO_EXCEPTION_PREFIX + e.getMessage());
     }
+
+    kp.setPrincipalName(null);
+    kp.setKeytab(null);
+    try {
+      ServiceApiUtil.validateKerberosPrincipal(app.getKerberosPrincipal());
+    } catch (NullPointerException e) {
+        Assert.fail(NO_EXCEPTION_PREFIX + e.getMessage());
+    }
   }
 
   @Test
@@ -718,6 +708,59 @@ public class TestServiceApiUtil extends ServiceTestUtils {
     }
   }
 
+  @Test(timeout = 1500)
+  public void testNoServiceDependencies() {
+    Service service = createExampleApplication();
+    Component compa = createComponent("compa");
+    Component compb = createComponent("compb");
+    service.addComponent(compa);
+    service.addComponent(compb);
+    List<String> dependencies = new ArrayList<String>();
+    service.setDependencies(dependencies);
+    ServiceApiUtil.checkServiceDependencySatisified(service);
+  }
+
+  @Test
+  public void testServiceDependencies() {
+    Thread thread = new Thread() {
+      @Override
+      public void run() {
+        Service service = createExampleApplication();
+        Component compa = createComponent("compa");
+        Component compb = createComponent("compb");
+        service.addComponent(compa);
+        service.addComponent(compb);
+        List<String> dependencies = new ArrayList<String>();
+        dependencies.add("abc");
+        service.setDependencies(dependencies);
+        Service dependent = createExampleApplication();
+        dependent.setState(ServiceState.STOPPED);
+        ServiceApiUtil.checkServiceDependencySatisified(service);
+      }
+    };
+    thread.start();
+    try {
+      Thread.sleep(1000);
+    } catch (InterruptedException e) {
+    }
+    Assert.assertTrue(thread.isAlive());
+  }
+
+  @Test
+  public void testJvmOpts() throws Exception {
+    String invalidJvmOpts = "`ping -c 3 example.com`";
+    intercept(IllegalArgumentException.class,
+        "Invalid character in yarn.service.am.java.opts.",
+        () -> ServiceApiUtil.validateJvmOpts(invalidJvmOpts));
+    String validJvmOpts = "-Dyarn.service.am.java.opts=-Xmx768m "
+        + "-Djava.security.auth.login.config=/opt/hadoop/etc/jaas-zk.conf";
+    try {
+      ServiceApiUtil.validateJvmOpts(validJvmOpts);
+    } catch (Exception ex) {
+      fail("Invalid character in yarn.service.am.java.opts.");
+    }
+  }
+
   public static Service createExampleApplication() {
 
     Service exampleApp = new Service();
